package demoRunner;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Ajaa valitun paketin demoja
 * @author ji
 * @author vesal
 *
 */
public class DemoRunner {

    /**
     * Ajaa demon valitusta paketista, mutta ei aja kielletty luokkaa
     * @param args          komentorivin argumentit
     * @param packageName   paketin nimi josta etsitn luokkia
     * @param noRunName     kielletyn luokan nimi
     */
    public static void run(String args[], String packageName,
            String noRunName) {
        if (args.length > 1) {
            System.out.println(
                    "Anna parametrina vain yhden ajettavan demon nimi.\n"
                            + "Jos et anna parametreja, voit valita ajettavan demon "
                            + "listasta.");
            System.exit(1);
        }

        DemoRunner demo = new DemoRunner(packageName, noRunName);

        String msg = "";
        if (args.length == 0) {
            msg = demo.runInteractively();
        } else {
            msg = demo.runWithArgs(args);
        }
        System.out.println(msg);
    }

    private ClassList classes;


    /**
     * Alustaa ajaja-luokan
     * @param packageName paketin nimi josta ajettavia etsitn
     * @param noRunName mit luokkia ei saa ajaa.
     */
    public DemoRunner(String packageName, String noRunName) {
        classes = new ClassList(packageName, noRunName);
        classes.sort();
    }


    /**
     * Ajaetaan demo argumenttien perusteella
     * @param args merkijonotaulukko jossa ekana demon nimi ja sitten sen parametrit
     * @return virheilmoitus
     */
    public String runWithArgs(String[] args) {
        String sampleName = args[0];
        // String[] restOfArgs = java.util.Arrays
        // .copyOfRange(args, 1, args.length); // Ei lydy JDK <1.6:sta.
        String[] restOfArgs = new String[args.length - 1];
        for (int i = 0; i < args.length - 1; ++i) {
            restOfArgs[i] = args[i + 1];
        }
        Class<?> sampleClass = classes.get(sampleName);
        if (sampleClass == null)
            return "Demoa " + sampleName + " ei ole.";
        runDemo(sampleClass, restOfArgs);
        return "";
    }


    /**
     * Ajetaan demo kysymll kyttjlt ajettavan luokan nimi
     * @return virheilmoitus
     */
    public String runInteractively() {
        String resultMessage = "";

        System.out.println("Valitse ajettava demo");

        int number = 0;
        for (Class<?> aClass : classes) {
            System.out.printf("  %d. %s\n", number++, aClass.getName());
        }

        BufferedReader consoleInputStream = new BufferedReader(
                new InputStreamReader(System.in));

        while (true) {
            try {
                System.out.print("Anna ajettavan demon numero: ");
                String response = consoleInputStream.readLine();
                if (response.equals("")) {
                    break;
                }
                int index = Integer.parseInt(response);
                Class<?> sampleClass = classes.get(index);
                runDemo(sampleClass, null);
                resultMessage = "";
                break;
            } catch (IOException e) {
                resultMessage = "Demoa ei voitu ajaa: "
                        + e.getLocalizedMessage();
                System.err.println(resultMessage);
            } catch (NumberFormatException e) {
                System.out.println("Anna numero 0..." + (classes.size() - 1));
                resultMessage = "";
            } catch (IndexOutOfBoundsException e) {
                System.out.println("Anna numero 0..." + (classes.size() - 1));
                resultMessage = "";
            }
        }

        return resultMessage;
    }


    private void runDemo(Class<?> sampleClass, String[] args) {
        try {
            Object objectParameters[] = { new String[] {} };
            Class<?> classParameters[] = { objectParameters[0].getClass() };
            Method mainMethod = sampleClass.getDeclaredMethod("main",
                    classParameters);
            mainMethod.invoke(null, (Object) args);
        } catch (SecurityException e) { // Virheet sivuutetaan, sill
            e.printStackTrace(System.err); // niist ei ole haittaa.
        } catch (IllegalArgumentException e) { // Tulostetaan ne kuitenkin
            e.printStackTrace(System.err); // virhevirtaan.
        } catch (NoSuchMethodException e) {
            e.printStackTrace(System.err);
        } catch (IllegalAccessException e) {
            e.printStackTrace(System.err);
        } catch (InvocationTargetException e) {
            e.printStackTrace(System.err);
        }
    }

    /**
     * Luokka ajettavista demoista 
     * @author ji
     */
    public static class ClassList implements Iterable<Class<?>> {
        private List<Class<?>> classes;
        private String packageOfClasses;
        private String noRunName;


        /**
         * Tutkitaan onko tiedosto sallittu tiedosto.
         * @param name      tiedoston nimi
         * @param noRunName1 nimi joka ei saa olla
         * @return true jos nimi on sallittu
         */
        private boolean isAllowed(String name, String noRunName1) {
            return !name.endsWith(noRunName1 + ".class");
        }


        /**
         * Alustetaan ajettavien luokkien lista
         * @param packageName paketti josta ajettavia etsitn
         * @param noRunName luokka jota ei saa ajaa
         */
        public ClassList(String packageName, String noRunName) {
            classes = new ArrayList<Class<?>>();
            add(packageName, noRunName);
        }


        /**
         * Listn ajettavien demojen joukkoon uusia demoja
         * @param packageName paketti josta luokkia estitn
         * @param noRunName1 nimi joita ei saa list
         */
        public final void add(String packageName, String noRunName1) {
            String thisPackageName = DemoRunner.class.getPackage().getName();
            String thisName = DemoRunner.class.getSimpleName();
            this.noRunName = noRunName1;
            packageOfClasses = packageName;
            URL demoURL = DemoRunner.class.getResource(thisName + ".class");
            File classFile = new File(demoURL.toExternalForm());
            String pathnameOfClass = classFile.getParentFile().getPath();

            String[] packagePath = pathnameOfClass.split("!")[0].split(":");
            String thisPackage = packagePath[packagePath.length - 1];
            String packageRoot = thisPackage.substring(0,
                    thisPackage.length() - thisPackageName.length());

            if (packagePath[0].equals("jar")/* packageFile.isFile() */) {
                String packageFileName = thisPackage;
                File packageFile = new File(packageFileName);
                classesInJar(packageFile);
            } else /* if (fff.isDirectory()) */ {
                String packageFileName = packageRoot + File.separator
                        + packageName;
                File packageFile = new File(packageFileName);
                classesInDirectory(packageFile);
            }
        }


        private void classesInDirectory(final File directory) {
            File[] files = directory.listFiles(new FilenameFilter() {
                @SuppressWarnings("synthetic-access")
                @Override
                public boolean accept(File dir, String name) {
                    return dir.equals(directory) && name.endsWith(".class")
                            && !name.contains("$")
                            && isAllowed(name, noRunName);
                }
            });
            for (File file : files) {
                loadSampleClass(file);
            }
        }


        private void classesInJar(File jarFile) {
            @SuppressWarnings("resource")
            JarInputStream jarStream = null;
            try {
                jarStream = new JarInputStream(new FileInputStream(jarFile));
                JarEntry jarEntry = jarStream.getNextJarEntry();
                while (jarEntry != null) {
                    if (jarEntry.toString().endsWith(".class")
                            && isAllowed(jarEntry.toString(), noRunName)) {

                        Pattern pp = Pattern.compile(
                                packageOfClasses + "/([\\w\\d_]+)\\.class");
                        Matcher mm = pp.matcher(jarEntry.toString());
                        if (mm.matches()) {
                            loadSampleClass(mm.group(1));
                        }
                    }
                    try {
                        jarEntry = jarStream.getNextJarEntry();
                    } catch (IOException e) {
                        // Ei vlitet tst, vaan yritetn uudelleen
                    }
                }
            } catch (FileNotFoundException e) { // Virhetilanteessa palautuu
                                                // tyhj
                // lista
                e.printStackTrace(System.err); // tai luetut alkiot, joten ei
                // vlitet poikkeutuksista.
            } catch (IOException e) { // Tulostetaan ne kuitenkin talteen
                // virhevirtaan.
                e.printStackTrace(System.err);
            } finally {
                if (jarStream != null)
                    try {
                        jarStream.close();
                    } catch (IOException e) { // mittp sit tehtisiin
                    }
            }
        }


        /**
         * Returns an iterator over the classes in this list in proper sequence.
         * @return an iterator over the elements in proper sequence
         */
        @Override
        public Iterator<Class<?>> iterator() {
            return classes.iterator();
        }


        /**
         * Lajittelee luokat jrjestykseen nimen perusteella
         */
        public void sort() {
            java.util.Collections.sort(classes, new Comparator<Class<?>>() {
                @Override
                public int compare(Class<?> o1, Class<?> o2) {
                    return o1.getName().compareTo(o2.getName());
                }
            });
        }


        /**
         * Finds a sample class by name.
         * 
         * @param sampleName
         *            name of sample class to be found.
         * @return sample class, or null if not found.
         */
        public Class<?> get(String sampleName) {
            for (Class<?> aClass : this) {
                if (aClass.getName().endsWith(sampleName)) {
                    return aClass;
                }
            }
            return null;
        }


        /**
         * Finds a Sample class by index.
         * 
         * @param index
         *            index of a Sample class to return
         * @return a Sample class, or null if not found
         * @exception IndexOutOfBoundsException,
         *                if the index is out of range.
         */
        public Class<?> get(int index) throws IndexOutOfBoundsException {
            return classes.get(index);
        }


        /**
         * The number of classes in this list, or Integer.MAX_VALUE, if the list
         * contains more than Integer.MAX_VALUE classes.
         * 
         * @return the number of elements in the class list
         */
        public int size() {
            return classes.size();
        }


        private Class<?> loadSampleClass(File file) {
            String filename = file.getName();
            return loadSampleClass(
                    filename.substring(0, filename.lastIndexOf(".")));
        }


        private Class<?> loadSampleClass(String fname) {
            Class<?> candidateClass = null;

            try {
                candidateClass = java.lang.Class.forName(
                        packageOfClasses + "." + fname, true,
                        ClassLoader.getSystemClassLoader());
            } catch (ClassNotFoundException e) {
                return null;
            }
            classes.add(candidateClass);
            return candidateClass;
        }

    }

}
