package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka akseleille
 * @author Markus Kivioja
 *
 */
public class Axis extends BasicShape {

    /** X koordinaattia kuvaava vakio */
    public final static int X = 0;

    /** Y koordinaattia kuvaava vakio */
    public final static int Y = 1;

    /** Z koordinaattia kuvaava vakio */
    public final static int Z = 2;

    private Line axisX, axisY, axisZ;
    private boolean showXscale, showYscale;
    private double xLength, yLength, zLength;
    private int xScalePoints, yScalePoints;
    private FillPolygon xArrowHead, yArrowHead;
    private double arrowHeadSize = 0.01;


    public Axis() {
        super();
        this.xArrowHead = new FillPolygon();
        this.yArrowHead = new FillPolygon();
        this.xScalePoints = 0;
        this.yScalePoints = 0;
        this.xLength = 0;
        this.yLength = 0;
        this.zLength = 0;
        this.axisX = new Line();
        this.axisY = new Line();
        this.axisZ = new Line();
        this.showXscale = true;
        this.showYscale = true;
    }


    /**
     * Luo uudet akselit
     * @param xLength x-akselin pituus
     * @param yLength y-akselin pituus
     * @param zLength z-akselin pituus
     * @param x akselien leikkauspisteen x-koordinaatti
     * @param y akselien leikkauspisteen y-koordinaatti
     * @param z akselien leikkauspisteen z-koordinaatti
     */
    public Axis(double xLength, double yLength, double zLength, double x,
            double y, double z) {
        super();
        double[][] points = { { xLength / 2, -xLength * arrowHeadSize },
                { xLength / 2, xLength * arrowHeadSize },
                { xLength / 2 + (xLength * 2 * arrowHeadSize), 0 } };
        this.xArrowHead = new FillPolygon(points);
        double[][] points2 = { { -yLength * arrowHeadSize, yLength / 2 },
                { yLength * arrowHeadSize, yLength / 2 },
                { 0, yLength / 2 + (yLength * 2 * arrowHeadSize) } };
        this.yArrowHead = new FillPolygon(points2);
        this.xScalePoints = 10;
        this.yScalePoints = 10;
        this.xLength = xLength;
        this.yLength = yLength;
        this.zLength = zLength;
        this.axisX = new Line(x - xLength / 2, y, z, x + xLength / 2, y, z);
        this.axisY = new Line(x, y - yLength / 2, z, x, y + yLength / 2, z);
        this.axisZ = new Line(x, y, z - this.zLength / 2, x, y,
                z + this.zLength / 2);
        this.showXscale = true;
        this.showYscale = true;
    }


    /**
     * Luo uudet akselit
     * @param xLength x-akselin pituus
     * @param yLength y-akselin pituus
     * @param zLength z-akselin pituus
     */
    public Axis(double xLength, double yLength, double zLength) {
        this(xLength, yLength, zLength, 0, 0, 0);
    }


    /**
     * Asettaa x-akselin asteikon nkyvksi
     * @param showXscale asteikko nkyv/piilotettu
     */
    public void showXscale(boolean showXscale) {
        this.showXscale = showXscale;
        this.redraw();
    }


    /**
     * Asettaa y-akselin asteikon nkyvksi
     * @param showYscale asteikko nkyv/piilotettu
     */
    public void showYscale(boolean showYscale) {
        this.showYscale = showYscale;
        this.redraw();
    }


    /**
     * Asettaa akselien pituudet
     * @param xLength x-akselin pituus
     * @param yLength y-akselin pituus
     * @param zLength z-akselin pituus
     */
    public void setLengths(double xLength, double yLength, double zLength) {
        this.xLength = xLength;
        this.yLength = yLength;
        this.zLength = zLength;
        this.axisX.setPoints(-this.xLength / 2, 0, 0, this.xLength / 2, 0, 0);
        this.axisY.setPoints(0, -this.yLength / 2, 0, 0, this.yLength / 2, 0);
        this.axisZ.setPoints(0, 0, -this.zLength / 2, 0, 0, this.zLength / 2);
        this.redraw();
    }


    /**
     * Asettaa x-akselin asteikon lukujen mrn
     * @param xScalePoints asteikon lukujen mr
     */
    public void setXscalePoints(int xScalePoints) {
        this.xScalePoints = xScalePoints;
        this.redraw();
    }


    /**
     * Asettaa y-akselin asteikon lukujen mrn
     * @param yScalePoints asteikon lukujen mr
     */
    public void setYscalePoints(int yScalePoints) {
        this.yScalePoints = yScalePoints;
        this.redraw();
    }


    /**
     * Asettaa akselien nuolenpiden koon
     * @param arrowHeadSize nuolenpiden koko
     */
    public void setArrowHeadSize(double arrowHeadSize) {
        this.arrowHeadSize = arrowHeadSize;
        this.redraw();
    }


    /**
     * Pyoristaa luvun halutulla desimaalimaaralla.
     * Vain sisaiseen kayttoon
     * @param number pyoristettava luku
     * @param digits desimaalien maara
     * @return pyoristetty luku
     */
    private double round(double number, int digits) {
        return ((int) (number * Math.pow(10, digits)) / Math.pow(10, digits));
    }


    @Override
    protected void drawShape(Graphics g, Matrix a) {
        Text value = null;
        Line l = new Line(0, 0, 0, 0);
        double dx = xLength / xScalePoints / 50;
        if (this.showXscale) {
            for (double i = -xLength / xScalePoints; i >= -xLength
                    / 2; i -= xLength / xScalePoints) {
                value = new Text(round(i, 2) + "", i, 0);
                value.setTranslate(0, -1.2);
                value.draw(g, a);
                l.setPoints(i, -dx, 0, i, dx, 0);
                l.draw(g, a);
            }
            for (double i = xLength / xScalePoints; i <= xLength
                    / 2; i += xLength / xScalePoints) {
                value = new Text(round(i, 2) + "", i, 0);
                value.setTranslate(0, -1.2);
                value.draw(g, a);
                l.setPoints(i, -dx, 0, i, dx, 0);
                l.draw(g, a);
            }
        }
        if (this.showYscale) {
            for (double i = -yLength / yScalePoints; i >= -yLength
                    / 2; i -= yLength / yScalePoints) {
                value = new Text(" " + round(i, 2) + "", 0, i);
                value.setTranslate(-1.2, 0);
                value.draw(g, a);
                l.setPoints(-dx, i, 0, dx, i, 0);
                l.draw(g, a);
            }
            for (double i = yLength / yScalePoints; i <= yLength
                    / 2; i += yLength / yScalePoints) {
                value = new Text(" " + round(i, 2) + "", 0, i);
                value.setTranslate(-1.2, 0);
                value.draw(g, a);
                l.setPoints(-dx, i, 0, dx, i, 0);
                l.draw(g, a);
            }
        }
        this.axisX.setColor(getColor());
        this.axisY.setColor(getColor());
        this.axisZ.setColor(getColor());
        this.axisX.draw(g, a);
        this.axisY.draw(g, a);
        this.axisZ.draw(g, a);
        this.xArrowHead.draw(g, a);
        this.yArrowHead.draw(g, a);
    }

}
