package fi.jyu.mit.graphics;

import java.awt.Graphics;

/**
 * Luokka R->R funktion kuvaajan piirtmiseksi.
 * @author Vesa Lappalainen
 *
 */
public class FunctionMapRR extends BasicShape {
    private double xmin, xmax;
    private FunctionRR f;
    private double z = 0.0;


    /**
     * Luo uuden R->R funktion kuvaajan
     * @param f piirrettv funktio
     * @param xmin x minimi
     * @param xmax x maksimi
     * @param z z-koordinaatin arvo
     */
    public FunctionMapRR(FunctionRR f, double xmin, double xmax, double z) {
        this(f, xmin, xmax);
        this.z = z;
    }


    /**
     * Luo uuden R->R funktion kuvaajan xy-tasoon
     * @param f piirrettv funktio
     * @param xmin x minimi
     * @param xmax x maksimi
     */
    public FunctionMapRR(FunctionRR f, double xmin, double xmax) {
        super();
        this.f = f;
        this.xmin = xmin;
        this.xmax = xmax;
    }


    @Override
    protected void drawShape(Graphics g, Matrix a) {
        double dx = (xmax - xmin) / 1000;
        if (dx == 0)
            dx = 0.1;
        SPoint sp = new SPoint(0, 0);
        Vector vr = new Vector(); // NOPMD
        int x1, y1, x2, y2;

        a.transform(vr.set(xmin, f.f(xmin), z), sp);
        x1 = sp.getX();
        y1 = sp.getY();

        for (double x = xmin + dx; x <= xmax; x += dx) {
            a.transform(vr.set(x, f.f(x), z), sp);
            x2 = sp.getX();
            y2 = sp.getY();
            g.drawLine(x1, y1, x2, y2);
            x1 = x2;
            y1 = y2;
        }
        a.transform(vr.set(xmax, f.f(xmax), z), sp);
        x2 = sp.getX();
        y2 = sp.getY();
        g.drawLine(x1, y1, x2, y2);
    }


    /**
      * Etsii funktion f minimin vlill [x1,x2] askeltaen dx:n kokoisilla
      * askelilla.
      * @param f  funktio jonka minimi etsitn
      * @param x1 aloituspiste
      * @param x2 lopetuspiste
      * @param dx askellusvli
      * @return funktion minimi annetulla vlill
      * @example
      * <pre name="test">
      * funMin(SinFun.sin, 0.0,1.0,0.1)         ~~~  0.0;
      * funMin(SinFun.sin, 0.0,2*Math.PI,0.001) ~~~ -1.0;
      * </pre>
      */
    public static double funMin(FunctionRR f, double x1, double x2, double dx) {
        double fx, x, f_min = f.f(x2);
        for (x = x1; x < x2; x += dx) {
            fx = f.f(x);
            if (fx < f_min)
                f_min = fx;
        }
        return f_min;
    }


    /**
     * Etsii funktion f maksimin vlill [x1,x2] askeltaen dx:n kokoisilla
     * askelilla.
     * @param f  funktio jonka maskimia etsitn
     * @param x1 aloituspiste
     * @param x2 lopetuspiste
     * @param dx askellusvli
     * @return funktion maksimi annetulla vlill
     * @example
     * <pre name="test">
     * funMax(SinFun.sin,-1.0,0.0,0.1)      ~~~ 0.0;
     * funMax(SinFun.sin, 0.0,Math.PI,0.01) ~~~ 1.0;
     * </pre>
     */
    public static double funMax(FunctionRR f, double x1, double x2, double dx) {
        double fx, x, f_max = f.f(x2);
        for (x = x1; x < x2; x += dx) {
            fx = f.f(x);
            if (fx > f_max)
                f_max = fx;
        }
        return f_max;
    }
}
