package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka merkkipisteille.
 * Oliota voi kytt prototyyppin luomaan vastaavanlaisia
 * merkkej lis.
 * <pre>
 * Kytt esimerkiksi:
 *   Marker malliMerkki = new Marker(0,0,0.25).setMaxR(5).setType(Marker.CIRCLE);
 *   ...
 *   ikkuna.add(malliMerkki.create(x,y));  
 * </pre>
 * @author Vesa Lappalainen
 *
 */
public class Marker extends BasicShape {
    /**
     * Rajapinta markkerin piirtmiseksi
     * @author vesal
     * @version 19.10.2008
     */
    public interface MarkerInterface {
        /**
         * Piirt markerin ruudulle 
         * @param g grafiikkapinta johon piirretn
         * @param x markerin x-keskipiste skaalattuna kokonaislukuna
         * @param y markerin y-keskipiste skaalattuna kokonaislukuna
         * @param r markerin sden skaalattuna kokonaislukuna
         */
        public void draw(Graphics g, int x, int y, int r);
    }

    /**
     * Valmis luokka ympyrmerkkien piirtmist varten
     */
    public static class Circle implements MarkerInterface {
        /**
         * @see fi.jyu.mit.graphics.Marker.MarkerInterface#draw(java.awt.Graphics, int, int, int)
         */
        @Override
        public void draw(Graphics g, int x, int y, int r) {
            g.drawOval(Math.round(x - r), Math.round(y - r), 2 * r, 2 * r);
        }
    }

    /**
     * Valmis luokka Plus-merkkien piirtmist varten
     */
    public static class Plus implements MarkerInterface {
        /**
         * @see fi.jyu.mit.graphics.Marker.MarkerInterface#draw(java.awt.Graphics, int, int, int)
         */
        @Override
        public void draw(Graphics g, int x, int y, int r) {
            g.drawLine(x - r - 1, y, x + r + 1, y);
            g.drawLine(x, y + r + 1, x, y - r - 1);
        }
    }

    /**
     * Valmis luokka X-merkkien piirtmist varten
     */
    public static class Cross implements MarkerInterface {
        /**
         * @see fi.jyu.mit.graphics.Marker.MarkerInterface#draw(java.awt.Graphics, int, int, int)
         */
        @Override
        public void draw(Graphics g, int x, int y, int r) {
            g.drawLine(x - r - 1, y - r - 1, x + r + 1, y + r + 1);
            g.drawLine(x - r - 1, y + r + 1, x + r + 1, y - r - 1);
        }
    }

    /** Ympyrn piirtv markkeri  */
    public static final MarkerInterface CIRCLE = new Circle();
    /** Plussan piirtv markkeri  */
    public static final MarkerInterface PLUS = new Plus();
    /** X:n piirtv piirtv markkeri  */
    public static final MarkerInterface CROSS = new Cross();

    private MarkerInterface marker = CIRCLE;
    private final RPoint center;
    private double r;
    private int maxR = Integer.MAX_VALUE;
    private int minR = 1;


    /**
     * Luo merkkipisteen
     * @param x merkin keskipisteen x-koordinaatti
     * @param y merkin keskipisteen y-koordinaatti
     * @param r ympyrn sde
     */
    public Marker(double x, double y, double r) {
        super();
        this.center = new RPoint(x, y);
        this.r = r;
        initialize();
    }


    /**
     * Luo merkkipisteen
     * @param x merkin keskipisteen x-koordinaatti
     * @param y merkin keskipisteen y-koordinaatti
     * @param r ympyrn sde
     * @param type piirrettvn merkin tyyppi
     */
    public Marker(double x, double y, double r, MarkerInterface type) {
        this(x, y, r);
        marker = type;
    }


    /**
     * Luo uuden ympyrn
     * @param x ympyrn keskipisteen x-koordinaatti
     * @param y ympyrn keskipisteen y-koordinaatti
     * @param z ympyrn keskipisteen z-koordinaatti
     * @param r ympyrn sde
     */
    public Marker(double x, double y, double z, double r) {
        super();
        this.center = new RPoint(x, y, z);
        this.r = r;
        initialize();
    }


    /**
     * Luo uuden ympyrn
     * @param x ympyrn keskipisteen x-koordinaatti
     * @param y ympyrn keskipisteen y-koordinaatti
     * @param z ympyrn keskipisteen z-koordinaatti
     * @param r ympyrn sde
     * @param type piirrettvn merkin tyyppi
     */
    public Marker(double x, double y, double z, double r,
            MarkerInterface type) {
        this(x, y, z, r);
        marker = type;
    }


    /**
     * Alustaa markerin asetukset
     */
    private void initialize() {
        //
    }


    /**
     * Asetetaan merkille uusi sde.  Negatiivinen sde
     * tarkoittaa mr pikselein.  Positiivinen alueen
     * koordinaatteina.
     * @param r uusi steen arvo
     * @return tm olio ketjuttamista varten
     */
    public Marker setR(double r) {
        this.r = r;
        redraw();
        return this;
    }


    /**
     * Asetetaan uusi tapa piirt merkki
     * @param type merkin piirtj
     * @return tm olio ketjuttamista varten
     */
    public Marker setType(MarkerInterface type) {
        this.marker = type;
        redraw();
        return this;
    }


    /**
     * Asetetaan merkin maksimikoko pikselein
     * @param maxR merkin maksimikoko pikselein
     * @return tm olio ketjuttamista varten
     */
    public Marker setMaxR(int maxR) {
        this.maxR = maxR;
        redraw();
        return this;
    }


    /**
     * Asetetaan merkin maksimikoko pikselein
     * @param minR merkin maksimikoko pikselein
     * @return tm olio ketjuttamista varten
     */
    public Marker setMinR(int minR) {
        this.minR = minR;
        redraw();
        return this;
    }


    /**
     * Piirretn merkki
     * @see fi.jyu.mit.graphics.BasicShape#drawShape(java.awt.Graphics, fi.jyu.mit.graphics.Matrix)
     */
    @Override
    protected void drawShape(Graphics g, Matrix a) {
        Vector rp = new Vector(); // NOPMD, luulee vrksi vektoriksi
        SPoint sp = new SPoint(0, 0);
        Vector spv = new Vector();
        double r0;
        int cx, cy, ri = (int) Math.round(-r);
        a.transform(center.getX(), center.getY(), center.getZ(), rp, sp);
        cx = sp.getX();
        cy = sp.getY();
        if (r >= 0) {
            // a.transform(center.getX()-r,center.getY(),center.getZ(),rp,sp);
            // ri = Math.abs(sp.getX()-cx); // TODO: tm pyristyy vlill
            // vrin
            a.transform(r, 0, 0, rp, spv); // TODO: ei ota perpektiivi huomioon
            r0 = spv.a(0);
            a.transform(0, 0, 0, rp, spv);
            ri = (int) Math.round(Math.abs(r0 - spv.a(0)));
        }
        if (ri > maxR)
            ri = maxR;
        if (ri < minR)
            ri = minR;
        marker.draw(g, cx, cy, ri);
    }


    /**
     * Metodi joka luo vastaavilla arvoilla olevan merkin
     * @param x merkin keskipisteen x-koordinaatti
     * @param y merkin keskipisteen y-koordinaatti
     * @param z merkin keskipisteen z-koordinaatti
     * @return uusi merkki
     */
    public Marker create(double x, double y, double z) {
        Marker m = new Marker(x, y, z, r, marker);
        m.setColor(getColor());
        m.minR = minR;
        m.maxR = maxR;
        return m;
    }


    /**
     * Metodi joka luo vastaavilla arvoilla olevan merkin
     * @param x merkin keskipisteen x-koordinaatti
     * @param y merkin keskipisteen y-koordinaatti
     * @return uusi merkki
     */
    public Marker create(double x, double y) {
        return create(x, y, 0);
    }

}
